@@ -14,21 +14,27 @@
 """
 Facilities for managing and reporting errors.
 """
+# isort: STDLIB
 import os
 import sys
 
+# isort: THIRDPARTY
 import dbus
 
-from dbus_client_gen import DbusClientMissingPropertyError
-from dbus_client_gen import DbusClientMissingSearchPropertiesError
-from dbus_client_gen import DbusClientUniqueResultError
+# isort: FIRSTPARTY
+from dbus_client_gen import (
+    DbusClientMissingPropertyError,
+    DbusClientMissingSearchPropertiesError,
+    DbusClientUniqueResultError,
+)
 
-from ._actions import BLOCKDEV_INTERFACE
-from ._actions import FILESYSTEM_INTERFACE
-from ._actions import POOL_INTERFACE
-
-from ._errors import StratisCliEngineError
-from ._errors import StratisCliUnknownInterfaceError
+from ._actions import BLOCKDEV_INTERFACE, FILESYSTEM_INTERFACE, POOL_INTERFACE
+from ._errors import (
+    StratisCliEngineError,
+    StratisCliIncoherenceError,
+    StratisCliUnknownInterfaceError,
+    StratisCliUserError,
+)
 
 _DBUS_INTERFACE_MSG = (
     "The version of stratis you are running expects a different "
@@ -74,7 +80,8 @@ def get_errors(exc):
 
 
 # pylint: disable=too-many-return-statements
-def interpret_errors(errors):
+# pylint: disable=too-many-branches
+def _interpret_errors(errors):
     """
     Laboriously add best guesses at the cause of the error, based on
     developer knowledge and possibly further information that is gathered
@@ -85,9 +92,9 @@ def interpret_errors(errors):
     :returns: None if no interpretation found, otherwise str
     """
     # pylint: disable=fixme
-    # TODO: This method is extremely rudimentary. It should not be extended
-    # using exactly the structure it has now.
-    try:
+    # FIXME: remove no coverage pragma when adequate testing for CLI output
+    # exists.
+    try:  # pragma: no cover
         # Inspect top-most error after StratisCliActionError
         error = errors[1]
 
@@ -113,10 +120,7 @@ def interpret_errors(errors):
         if isinstance(error, DbusClientMissingPropertyError):  # pragma: no cover
             return _DBUS_INTERFACE_MSG
 
-        # pylint: disable=fixme
-        # FIXME: remove no coverage pragma when adequate testing for CLI
-        # output exists.
-        if isinstance(error, StratisCliEngineError):  # pragma: no cover
+        if isinstance(error, StratisCliEngineError):
             fmt_str = (
                 "stratisd failed to perform the operation that you "
                 "requested. It returned the following information via "
@@ -124,17 +128,32 @@ def interpret_errors(errors):
             )
             return fmt_str % error
 
+        if isinstance(error, StratisCliUserError):
+            fmt_str = "It appears that you issued an unintended command: %s"
+            return fmt_str % error
+
+        # An incoherence error should be pretty untestable. It can arise
+        # * in the case of a stratisd bug. We would expect to fix that very
+        # soon, so should not have a test in that case.
+        # * in the case where another client of stratisd is running and alters
+        # state while a command is being executed. This could be tested for,
+        # but only with considerable difficulty, so we choose not to test.
+        if isinstance(error, StratisCliIncoherenceError):  # pragma: no cover
+            fmt_str = (
+                "stratisd reported that it did not execute every action "
+                "that it would have been expected to execute as a result "
+                "of the command that you requested: %s"
+            )
+            return fmt_str % error
+
         # Inspect lowest error
         error = errors[-1]
 
-        # pylint: disable=fixme
-        # FIXME: remove no coverage pragma when adequate testing for CLI
-        # output exists.
         if (
             # pylint: disable=bad-continuation
             isinstance(error, dbus.exceptions.DBusException)
             and error.get_dbus_name() == "org.freedesktop.DBus.Error.AccessDenied"
-        ):  # pragma: no cover
+        ):
             return "Most likely stratis has insufficient permissions for the action requested."
         # We have observed two causes of this problem. The first is that
         # stratisd is not running at all. The second is that stratisd has not
@@ -146,14 +165,11 @@ def interpret_errors(errors):
         ):
             return "Most likely stratis is unable to connect to the stratisd D-Bus service."
 
-        # pylint: disable=fixme
-        # FIXME: remove no coverage pragma when adequate testing for CLI
-        # output exists.
         if (
             # pylint: disable=bad-continuation
             isinstance(error, dbus.exceptions.DBusException)
             and error.get_dbus_name() == "org.freedesktop.DBus.Error.NoReply"
-        ):  # pragma: no cover
+        ):
             fmt_str = (
                 "stratis attempted communication with the daemon, stratisd, "
                 "over the D-Bus, but stratisd did not respond in the allowed time."
@@ -183,7 +199,7 @@ def handle_error(err):
 
     errors = list(get_errors(err))
 
-    explanation = interpret_errors(errors)
+    explanation = _interpret_errors(errors)
 
     # The goal is to have an explanation for every error chain. If there is
     # none, then this will rapidly be fixed, so it will be difficult to
